@@ -12,6 +12,9 @@ typedef struct ITEXT{
 
 	char *utf8;
 
+	int x_off;
+	int y_off;
+
 	int ej;
 	int chu;
 	int bouten;
