@@ -82,6 +82,9 @@ static ITEXT_T *itext_data_alloc()
 	it->c1 = ' ';
 	it->c2 = ' ';
 
+	it->x_off = 0;
+	it->y_off = 0;
+
 	it->ej = 0;
 	it->chu = 0;
 	it->bouten = 0;
