@@ -11,10 +11,11 @@
 #ifndef __XTEXT_
 
 #define XTEXT_NAME 	"xjp-text"
-#define XTEXT_VERSION 	"1.0.5"
+#define XTEXT_VERSION 	"1.1.0"
 #define CONF_DIR_NAME 	".xtext"
 #define CONF_FILE_NAME 	"xtext.conf"
 #define SIORI_FILE_NAME "xtext.siori"
+#define OFFSET_FILE_NAME "xtext.offset"
 
 #define XJP_OK  "xjp-ok"
 #define XJP_CANCEL  "xjp-cancel"
@@ -176,6 +177,19 @@ typedef struct {
 	book_data_t *book;
 
 }char_data_t;
+typedef struct XOFFSET {
+	char font[128];
+	unsigned int c1;
+	unsigned int c2;
+
+	int anti;
+
+	int x_off;
+	int y_off;
+
+	struct XOFFSET *next;
+	struct XOFFSET *prev;
+}offset_data_t;
 
 typedef struct SUBWINDOWDATA{
 	char line_str[128];
@@ -230,7 +244,7 @@ char * xtextSubWindow(Display *,my_window_t,sub_window_t *,sub_window_t *);
 
 int xtextSubWindow_Message(Display *,my_window_t,char *);
 
-int xtext_get_charsize(Display *,my_window_t,IBUF_T *,char_data_t *);
+int xtext_get_charsize(Display *,my_window_t *,IBUF_T *,char_data_t *);
 
 int xtext_next_kuten(ITEXT_T *);
 
@@ -255,3 +269,10 @@ char *xtext_Selector_Window(char *,int);
 int xtext_font_J2E(char *,char *);
 
 int parse_xlfd_No(char *,char *,int);
+
+int xtext_offt_data(offset_data_t **,offset_data_t **);
+
+int xtext_offt_set(offset_data_t *,offset_data_t *,xtext_conf_t,offset_data_t **,offset_data_t **);
+
+int xtext_set_offset(IBUF_T *,IBUF_T *,offset_data_t *);
+
