@@ -12,6 +12,7 @@ static int XtextDrawString(	Display *disp,
 	GC gc;
 	XFontSet fs;
 	char text[3];
+	int xpad,ypad;
 
 	text[0] = it->c1;
 	text[1] = it->c2;
@@ -25,8 +26,10 @@ fprintf(stderr,"Normal_String : %s   ",text);
 	fs = win.fs;
 	if(it->chu) gc = win.rubi_gc;
 	if(it->small) fs = win.fs_s;
+	xpad = it->x_off;
+	ypad = it->y_off;
 
-	XmbDrawString(disp,win.win,fs,gc,x,y,text,strlen(text));
+	XmbDrawString(disp,win.win,fs,gc,x+xpad,y+ypad,text,strlen(text));
 
 #if debug
 fprintf(stderr,"END\n");
@@ -46,10 +49,14 @@ static int XtextDrawAntiString(	Display *disp,
 	XftColor color;
 	XGlyphInfo gi;
 	int xoff;
+	int xpad,ypad;
 
 	font = win.font;
 	color = win.fg_color;
 
+	xpad = it->x_off;
+	ypad = it->y_off;
+
 	if(it->chu) color = win.rubi_color;
 	if(it->small) font = win.font_s;
 
@@ -61,7 +68,7 @@ static int XtextDrawAntiString(	Display *disp,
 	}
 	
 
-	XftDrawStringUtf8(win.draw,&color,font,x+xoff,y,it->utf8,strlen(it->utf8));
+	XftDrawStringUtf8(win.draw,&color,font,x+xoff+xpad,y+ypad,it->utf8,strlen(it->utf8));
 #else
 
 	XtextDrawString(disp,win,char_d,it,x,y);
@@ -91,10 +98,12 @@ static int XtextDrawImageString(	Display *disp,
 	}
 	if(image_str == NULL) return -1;
 
-	x = x0;
-	y = y0 - char_d.charSize + 3;
 	charSize = char_d.charSize;
+	/*if(it->ej = 0) charSize = char_d.charSize_e;*/
 	if(it->small == 1) charSize = char_d.charSize_s;
+
+	x = x0;
+	y = y0 - char_d.charSize + 1;
 	XPutImage(disp,win.win,win.gc,image_str,0,0,x,y,charSize,charSize);
 
 
@@ -112,7 +121,15 @@ static int XtextDrawRubiString(	Display *disp,
 	int k_len=0,r_len=0;
 	int x,y;
 
-	if(win.rubi_fs == NULL) return -1;
+	if(win.anti == 0){
+		if(win.rubi_fs == NULL) return -1;
+	}else{
+#if HAVE_LIBXFT
+		if(win.font_rubi == NULL) return -1;
+#else
+		if(win.rubi_fs == NULL) return -1;
+#endif
+	}
 
 	for(rb=it;rb!=NULL;rb=rb->prev){
 		if(!(rb->c1 >= 0xb0)) break;
@@ -299,11 +316,9 @@ int xtext_Draw_Text(	Display *disp,
 
 	x = start_line;
 	y = start_low;
-
 #if debug
 fprintf(stderr,"Draw Start\n");
 #endif
-
 	for(ib=l_head;ib!=NULL;ib=ib->next){
 		for(it=ib->i_head;it!=NULL;it=it->next){
 			if(sc++ >= char_p->char_no) flag = 1;
