@@ -20,6 +20,7 @@ char *XJP_MESSAGE[] = {
 char *SELECTOR_BACKGROUND_COLOR = NULL;
 char *SELECTOR_FOREGROUND_COLOR = NULL;
 char *SELECTOR_SELECTFONT_COLOR = NULL;
+static int NPC = 0;
 
 extern int opterr;
 extern int optind;
@@ -36,8 +37,9 @@ static sub_window_t *si_head = NULL,*si_tail = NULL;
 static sub_window_t *help_head = NULL, *help_tail = NULL;
 
 static IBUF_T *line_head = NULL, *line_tail = NULL;
+static offset_data_t *offset_head = NULL, *offset_tail = NULL;
+static offset_data_t *off_head = NULL, *off_tail = NULL;
 static page_data_t *char_p;
-static int NPC = 0;
 
 static char_data_t char_d = {
 		-1,
@@ -118,12 +120,12 @@ static void _usage()
 
 	exit(1);
 }	
-static void _set_util()
+static void _set_utl()
 {
 	main_win.title_on = xconf.title_on;
 	main_win.anti = xconf.anti;
 }
-static void _set_window()
+static void _set_win()
 {
 	XSetWindowBackground(disp,main_win.win,main_win.bg_pixel);
 	XClearWindow(disp,main_win.win);
@@ -166,7 +168,7 @@ static void _x_drawable()
 	main_win.draw = XftDrawCreate(disp,main_win.win,main_win.visual,main_win.cmap);
 #endif
 }
-static void _set_color()
+static void _set_col()
 {
 	XColor bg0,bg1;
 	XColor fg0,fg1;
@@ -208,14 +210,14 @@ static void _set_color()
 	main_win.fg_color.color.alpha = 0xFFFF;
 
 	XftColorAllocName(disp,main_win.visual,main_win.cmap,xconf.rubi_font_color,&(main_win.rubi_color));
-	main_win.fg_color.color.alpha = 0xCCCC;
+	main_win.rubi_color.color.alpha = 0xCCCC;
 #endif
 
 	SELECTOR_BACKGROUND_COLOR = (char *)xconf.select_window_color;
 	SELECTOR_FOREGROUND_COLOR = (char *)xconf.select_font_color;
 	SELECTOR_SELECTFONT_COLOR = (char *)xconf.select_font_background_color;
 }
-static void _set_font()
+static void _set_fnt()
 {
 	char font_str[200],font_e[100];
 	char **miss,*def;
@@ -240,20 +242,23 @@ static void _set_font()
 	main_win.font_rubi = XftFontOpenXlfd(disp,0,xconf.rubi_font);
 #endif
 
+	xtext_offt_set(offset_head,offset_tail,xconf,&off_head,&off_tail);
+	xtext_set_offset(line_head,line_tail,off_head);
+
 }
-static void _set_char()
+static void _set_chr()
 {
 	char_d.xpad = xconf.line_space != -1 ? xconf.line_space:0;
 	char_d.ypad = xconf.row_space != -1 ? xconf.row_space:0;
 }
 static void _set_xconf_data()
 {
-	_set_color();
-	_set_window();
+	_set_utl();
 
-	_set_font();
-	_set_char();
-	_set_util();
+	_set_col();
+	_set_win();
+	_set_fnt();
+	_set_chr();
 
 	return ;
 }
@@ -273,10 +278,11 @@ static page_data_t *_set_redraw_text()
 {
 	if(line_head == NULL) return NULL;
 
-	xtext_get_charsize(disp,main_win,line_head,&char_d);
+	xtext_get_charsize(disp,&main_win,line_head,&char_d);
 
 	char_p = _get_page_data(char_d.head,NPC);
 
+
 	return char_p;
 }
 static int _init_book_data(char *book_name,int n)
@@ -300,6 +306,8 @@ fprintf(stderr,"main_bookname : %s %d\n",book_name,n);
 	line_head = NULL;
 	line_tail = NULL;
 	if(itextGetTextData(book_name,title,&line_head,&line_tail,xconf.ruby_on)<0) return -1;
+	xtext_set_offset(line_head,line_tail,off_head);
+
 #if debug
 fprintf(stderr,"main_booktitle : %s\n",title);
 #endif
@@ -436,7 +444,7 @@ static int _xtext_event()
 				else
 					xconf.title_on = 1;	
 
-				_set_util();
+				_set_utl();
 
 				NPC = xtext_Draw_Text(disp,main_win,line_head,char_d,char_p);
 			}
@@ -483,6 +491,8 @@ int main(int argc,char *argv[])
 	int loop = 1;
 
 	xtext_conf_data(&xconf,xconf_default);
+	xtext_offt_data(&offset_head,&offset_tail);
+	xtext_siori_data_read(&siori_head,&siori_tail);
 
 	opt_book_path = (char *)_get_option(argc,argv);
 
@@ -492,7 +502,6 @@ int main(int argc,char *argv[])
 
 	if(opt_book_path != NULL) _init_book_data(opt_book_path,0);
 
-	xtext_siori_data_read(&siori_head,&siori_tail);
 	while(loop){
 		while(XPending(disp)){
 			loop = _xtext_event();
