@@ -123,36 +123,124 @@ static void * _image_data_add(image_data_t **head,image_data_t **tail,image_data
 
 	return NULL;
 }
+int _xtext_max_wh(      Display *disp,
+                        my_window_t *win,
+                        char_data_t *char_d,
+                        ITEXT_T *it,
+                        int *xmax,
+                        int *ymax,
+ 			int *wmax,
+			int *hmax		)
+{
+        Pixmap pix;
+        XImage *im;
+
+        XftFont *font = NULL;
+        XftDraw *draw = NULL;
+        XftColor color;
+
+        XFontSet fs;
+        GC gc;
+        char temp[3];
+
+        unsigned long pixel;
+        int x,y,charSize,pad=5;
+        int x_min=9999,y_min=9999,x_max=0,y_max=0,w_max=0,h_max=0;
+
+        if(win->anti){
+                color = win->fg_color;
+                font = win->font;
+                charSize = char_d->charSize+pad;
+                if(it->small){
+                        font = win->font_s;
+                        charSize = char_d->charSize_s+pad;
+                }
+
+               	pix = XCreatePixmap(disp,DefaultRootWindow(disp),charSize,charSize,DefaultDepth(disp,0));
+               	XFillRectangle(disp,pix,win->bgc,0,0,charSize,charSize);
+
+                draw = XftDrawCreate(disp,pix,win->visual,win->cmap);
+                x = pad;
+                y = charSize-pad;
+                XftDrawStringUtf8(draw,&color,font,x,y,it->utf8,strlen(it->utf8));
+
+        }else{
+                temp[0] = it->c1;
+                temp[1] = it->c2;
+                temp[2] = '\0';
+                gc = win->gc;
+                fs = win->fs;
+                charSize = char_d->charSize+pad;
+                if(it->small){
+                        fs = win->fs_s;
+                        charSize = char_d->charSize_s+pad;
+                }
+                pix = XCreatePixmap(disp,DefaultRootWindow(disp),charSize,charSize,DefaultDepth(disp,0));
+                XFillRectangle(disp,pix,win->bgc,0,0,charSize,charSize);
+
+                x = pad; 
+                y = charSize-pad;
+                XmbDrawString(disp,pix,fs,gc,x,y,temp,strlen(temp));
+        }
+        im = XGetImage(disp,pix,0,0,charSize,charSize,0xFFFF,XYPixmap);
+
+        for(x=0;x<charSize;x++){
+                for(y=0;y<charSize;y++){
+                        pixel = XGetPixel(im,x,y);
+                        if(win->bg_pixel != pixel){
+                                if(x < x_min) x_min = x;
+                                if(x > x_max) x_max = x;
+				if(x > charSize - pad) w_max = y;
+                                if(y < y_min) y_min = y;
+                                if(y > y_max) y_max = y;
+				if(y > charSize - pad) h_max = y;
+                        }
+                }
+        }
+        if((*xmax = x_max - x_min) <= 0) *xmax = 1;
+        if((*ymax = y_max - y_min) <= 0) *ymax = 1;;
+	if((*wmax = w_max - (charSize - pad)) <= 0) *wmax = 0;
+	if((*hmax = h_max - (charSize - pad)) <= 0) *hmax = 0;
+
+        if(im) XDestroyImage(im);
+        if(pix) XFreePixmap(disp,pix);
+        if(draw) XftDrawDestroy(draw);
+
+
+        return 0;
+}
 static Pixmap _xtext_Draw_imageString_normal(	Display *disp,
-						my_window_t win,
+						my_window_t *win,
 						char_data_t *char_d,
 						ITEXT_T *it,
 						int type,
-						XImage **base_im,
-						int *charSize		)
+						XImage **base_im	)
 {
         Pixmap pix;
+	int charSize;
         int tx,ty;
+	int xpad,ypad;
 
 	XFontSet fs;
 	GC gc,gc1;
 	char text[3];
 
-	fs = win.fs;
-	gc1 = win.sgc;
-	gc = win.gc;
-	*charSize = char_d->charSize;
+	fs = win->fs;
+	gc1 = win->sgc;
+	gc = win->gc;
+	charSize = char_d->charSize;
 
 	if(it->chu == 1){
-	       	gc = win.rubi_gc;
+	       	gc = win->rubi_gc;
 	}
 	if(it->small == 1){
-	       	*charSize = char_d->charSize_s;
-		fs = win.fs_s;
+	       	charSize = char_d->charSize_s;
+		fs = win->fs_s;
 	}
 
 	if(it->ej == 0){
-		fs = win.fs_e;
+		/*charSize = char_d->charSize_e;*/
+		fs = win->fs_e;
 
 		text[0] = it->c1;
 		text[1] = '\0';
@@ -162,42 +250,47 @@ static Pixmap _xtext_Draw_imageString_normal(	Display *disp,
 		text[2] = '\0';
 	}
 
-        pix = XCreatePixmap(disp,DefaultRootWindow(disp),*charSize,*charSize,DefaultDepth(disp,0));
-        XFillRectangle(disp,pix,win.bgc,0,0,*charSize,*charSize);
-        *base_im = XGetImage(disp,pix,0,0,*charSize,*charSize,0xFFFF,XYPixmap);
+	xpad = it->x_off;
+	ypad = it->y_off;
 
+        pix = XCreatePixmap(disp,DefaultRootWindow(disp),charSize,charSize,DefaultDepth(disp,0));
+        XFillRectangle(disp,pix,win->bgc,0,0,charSize,charSize);
+        *base_im = XGetImage(disp,pix,0,0,charSize,charSize,0xFFFF,XYPixmap);
 
-      	if(type == 4){
-               	XRectangle ink,log;
+	if(it->ej == 1){
+		int myw,myh,myx,myy;
+		_xtext_max_wh(disp,win,char_d,it,&myw,&myh,&myx,&myy);
 
-               	XmbTextExtents(fs,text,strlen(text),&ink,&log);
+		if(type == 4){
+			tx = (charSize - myw)/2 - myx - 1;
+			ty = (charSize - myh)/2 + myh - myy + 1;
 
-               	if((log.width == ink.width) && (log.height == ink.height)){ 
-                       	tx = *charSize / 4;
-                       	ty = ink.height - (ink.height / 4);
-               	}else{ 
-                       	tx = (*charSize - ink.width) / 2;
-                       	ty = ink.height;
-               	}
+		}else if(type == 0){
+			tx = myx + 1;
+			ty = charSize - myy - 1;
+		}else{
+			tx = myx + 1;
+			ty = charSize - myy - 1;
+		}
 
-		XmbDrawString(disp,pix,fs,gc,tx,ty,text,strlen(text));
-       	}else{
+		XmbDrawString(disp,pix,fs,gc,tx+xpad,ty+ypad,text,strlen(text));
+	}else{
 		tx = 2;
-		ty = *charSize - 2;
-		XmbDrawString(disp,pix,fs,gc,2,*charSize-2,text,strlen(text));
-       	}
+		ty = charSize - 2;
+		XmbDrawString(disp,pix,fs,gc,tx+xpad,ty+ypad,text,strlen(text));
+	}
 
 	return pix;
 }
 static Pixmap _xtext_Draw_imageString_anti(	Display *disp,
-						my_window_t win,
+						my_window_t *win,
 						char_data_t *char_d,
 						ITEXT_T *it,
 						int type,
-						XImage **base_im,
-						int *charSize		)
+						XImage **base_im	)
 {
 	Pixmap pix;
+	int charSize;
 
 #if HAVE_LIBXFT
 
@@ -205,42 +298,48 @@ static Pixmap _xtext_Draw_imageString_anti(	Display *disp,
 	XftFont *font = NULL;
 	XftColor color;
 	int tx,ty;
+	int ypad;
+	int xpad;
 
-	font = win.font;
-	color = win.fg_color;
-	*charSize = char_d->charSize;
-	if(it->chu) color = win.rubi_color;
+	font = win->font;
+	color = win->fg_color;
+	charSize = char_d->charSize;
+	if(it->chu) color = win->rubi_color;
 	if(it->small){
-	       	font = win.font_s;
-		*charSize = char_d->charSize_s;
+	       	font = win->font_s;
+		charSize = char_d->charSize_s;
+	}
+	if(it->ej == 0){
+	       	font = win->font_e;
+		/*charSize = char_d->charSize_e;*/
 	}
-	if(it->ej == 0) font = win.font_e;
 
-        pix = XCreatePixmap(disp,DefaultRootWindow(disp),*charSize,*charSize,DefaultDepth(disp,0));
-        XFillRectangle(disp,pix,win.bgc,0,0,*charSize,*charSize);
-        *base_im = XGetImage(disp,pix,0,0,*charSize,*charSize,0xFFFF,XYPixmap);
+	xpad = it->x_off;
+	ypad = it->y_off;
 
-	draw = XftDrawCreate(disp,pix,win.visual,win.cmap);
+        pix = XCreatePixmap(disp,DefaultRootWindow(disp),charSize,charSize,DefaultDepth(disp,0));
+        XFillRectangle(disp,pix,win->bgc,0,0,charSize,charSize);
+        *base_im = XGetImage(disp,pix,0,0,charSize,charSize,0xFFFF,XYPixmap);
+	draw = XftDrawCreate(disp,pix,win->visual,win->cmap);
 
 	if(it->ej == 1){
-		if(type == 4 || type == 5){
-			XGlyphInfo gi;
-			int ypad = 0;
-			int xpad = 0;
-
-			if(type == 5) ypad = 2;
-
-			XftTextExtentsUtf8(disp,font,it->utf8,strlen(it->utf8),&gi);
-			tx = (*charSize - gi.width) / 2 + xpad;
-			ty = *charSize - (*charSize - gi.height) + (gi.y - gi.height) + ypad;
+		int myw,myh,myx,myy;
+		_xtext_max_wh(disp,win,char_d,it,&myw,&myh,&myx,&myy);
 
+		if(type == 4){
+			tx = (charSize - myw)/2 - myx - 1;
+			ty = (charSize - myh)/2 + myh - myy + 1;
 
+		}else if(type == 0){
+			tx = myx;
+			ty = charSize - myy - 1;
 		}else{
-			tx = 0;
-			ty = *charSize - 2;
+
+			tx = myx;
+			ty = charSize - myy - 1;
 		}
 			
-		XftDrawStringUtf8(draw,&color,font,tx,ty,it->utf8,strlen(it->utf8));
+		XftDrawStringUtf8(draw,&color,font,tx+xpad,ty+ypad,it->utf8,strlen(it->utf8));
 	}else{
 		char text[2];
 
@@ -248,7 +347,7 @@ static Pixmap _xtext_Draw_imageString_anti(	Display *disp,
 		text[1] = '\0';
 
 		tx = 2;
-		ty = *charSize - 2;
+		ty = charSize - 2;
 
 		XftDrawString8(draw,&color,font,tx,ty,text,strlen(text));
 	}
@@ -257,14 +356,14 @@ static Pixmap _xtext_Draw_imageString_anti(	Display *disp,
 	return pix;
 
 #else
-	pix = _xtext_Draw_imageString_normal(disp,win,char_d,it,type,base_im,charSize);
+	pix = _xtext_Draw_imageString_normal(disp,win,char_d,it,type,base_im);
 
 	return pix;
 #endif
 
 }
-static XImage * get_rotate_image(      Display *disp,
-                                	my_window_t win,
+static XImage * get_string_image(      Display *disp,
+                                	my_window_t *win,
 					char_data_t *char_d,
 					ITEXT_T *it,
                                 	int type        )
@@ -277,37 +376,38 @@ static XImage * get_rotate_image(      Display *disp,
         int x,y;
         int x0,x1,y0,y1;
 
-	if(win.anti == 0){
-		pix = _xtext_Draw_imageString_normal(disp,win,char_d,it,type,&base_im,&charSize);
+	if(win->anti == 0){
+		pix = _xtext_Draw_imageString_normal(disp,win,char_d,it,type,&base_im);
 
 	}else{
-		pix = _xtext_Draw_imageString_anti(disp,win,char_d,it,type,&base_im,&charSize);
+		pix = _xtext_Draw_imageString_anti(disp,win,char_d,it,type,&base_im);
 	}
+	charSize = char_d->charSize;
+	/*if(it->ej == 0) charSize = char_d->charSize_e;*/
+	if(it->small) charSize = char_d->charSize_s;
 #if debug 
 fprintf(stderr,"image charSize : %d\n",charSize);
 #endif
-
         char_im = XGetImage(disp,pix,0,0,charSize,charSize,0xFFFF,XYPixmap);
 
-        if(type == 0 || type == 1 || type == 4 || type == 5){ x0=0; x1=charSize; y0=0; y1=charSize;}
-
+	if(type == 0){ x0=0; x1=charSize; y0=0; y1=charSize;}
+	else if(type == 1){ x0=0; x1=charSize/2; y0=charSize/2; y1=charSize;}
 	else if(type == 2){ x0=0; x1=charSize/2+2; y0=0; y1=charSize/2;}
-
         else if(type == 3){ x0=charSize/2; x1=charSize; y0=0; y1=charSize/2;}
+	else if(type == 4 || type == 5){ x0=0; x1=charSize; y0=0; y1=charSize;}
 
         for(x=x0;x<x1;x++){
                 for(y=y0;y<y1;y++){
                         pixel = XGetPixel(char_im,x,y);
 
                         if(type == 0){
-                                XPutPixel(base_im,charSize-y-1,x,pixel);
+                                XPutPixel(base_im,charSize-y,x,pixel);
 
                         }else if(type == 1){
-                                XPutPixel(base_im,charSize-x-1,charSize-y-1,pixel);
+                                XPutPixel(base_im,x+charSize/2,y-charSize/2,pixel);
 
                         }else if(type == 2){
                                 XPutPixel(base_im,(charSize/2-2)+x-1,charSize/2+y+1,pixel);
-
                         }else if(type == 3){
                                 XPutPixel(base_im,x-charSize/2,charSize/2+y,pixel);
                         }else if(type == 4 || type == 5){
@@ -326,7 +426,7 @@ fprintf(stderr,"image END\n");
 
         return base_im;
 }
-static void * _image_data_insert(Display *disp,my_window_t win,char_data_t *char_d,ITEXT_T *it,int type)
+static void * _image_data_insert(Display *disp,my_window_t *win,char_data_t *char_d,ITEXT_T *it,int type)
 {
 	image_data_t *im;
 	int s_type;
@@ -352,7 +452,7 @@ static void * _image_data_insert(Display *disp,my_window_t win,char_data_t *char
 	if(type == 5) s_type = 4;
 	if(type == 6) s_type = 5;
 	
-	im->char_image = (XImage *)get_rotate_image(disp,win,char_d,it,s_type);
+	im->char_image = (XImage *)get_string_image(disp,win,char_d,it,s_type);
 
 	_image_data_add(&char_d->im_head,&char_d->im_tail,im);
 
@@ -409,11 +509,9 @@ static int _check_image_data(ITEXT_T *it)
 	}else if(it->c1 == 0xa4){
 		switch(it->c2){
 			case 0xa1:case 0xa3:case 0xa5:case 0xa7:case 0xa9:
-			case 0xe3:case 0xe5:case 0xe7:case 0xee:
+			case 0xe3:case 0xe5:case 0xe7:case 0xee:case 0xc3:
 				return 5;
 			break;
-			case 0xc3: return 6;
-			break;
 		}
 	}else if(it->c1 == 0xa5){
 		switch(it->c2){
@@ -426,29 +524,27 @@ static int _check_image_data(ITEXT_T *it)
 
 	return 0;
 }
-static int _xtext_get_normalfont_size(my_window_t win,int *charSize,int *charSize_s,int *charSize_e,int *charSize_r)
+static int _xtext_get_normalfont_size(my_window_t *win,int *charSize,int *charSize_s,int *charSize_e,int *charSize_r)
 {
 	char *str = "";
 	char *str_e = "W";
 	XRectangle ink,log;
 
-	XmbTextExtents(win.fs,str,strlen(str),&ink,&log);
+	XmbTextExtents(win->fs,str,strlen(str),&ink,&log);
 	*charSize = log.width >= log.height ? log.width+2 : log.height+2;
 
-	XmbTextExtents(win.fs_s,str,strlen(str),&ink,&log);
+	XmbTextExtents(win->fs_s,str,strlen(str),&ink,&log);
 	*charSize_s = log.width >= log.height ? log.width+2 : log.height+2;
 
-	XmbTextExtents(win.fs_e,str_e,strlen(str_e),&ink,&log);
+	XmbTextExtents(win->fs_e,str_e,strlen(str_e),&ink,&log);
 	*charSize_e = log.width+2;
 
-	if(win.rubi_fs != NULL){
-		XmbTextExtents(win.rubi_fs,str,strlen(str),&ink,&log);
-		*charSize_r = log.width >= log.height ? log.width : log.height;
-	}
+	XmbTextExtents(win->rubi_fs,str,strlen(str),&ink,&log);
+	*charSize_r = log.width >= log.height ? log.width : log.height;
 
 	return 0;
 }
-static int _xtext_get_antifont_size(Display *disp,my_window_t win,int *charSize,int *charSize_s,int *charSize_e,int *charSize_r)
+static int _xtext_get_antifont_size(Display *disp,my_window_t *win,int *charSize,int *charSize_s,int *charSize_e,int *charSize_r)
 {
 #if HAVE_LIBXFT
 	char *str = "";
@@ -461,20 +557,18 @@ static int _xtext_get_antifont_size(Display *disp,my_window_t win,int *charSize,
 	con = iconv_open("UTF-8","EUC-JP");
 	utf8 = (char *)euc2utf8(con,str,strlen(str));
 
-	XftTextExtentsUtf8(disp,win.font,utf8,strlen(utf8),&gi);
+	XftTextExtentsUtf8(disp,win->font,utf8,strlen(utf8),&gi);
 	*charSize = gi.width >= gi.height ? gi.width+2 : gi.height+2;
 
-	XftTextExtentsUtf8(disp,win.font_s,utf8,strlen(utf8),&gi);
+	XftTextExtentsUtf8(disp,win->font_s,utf8,strlen(utf8),&gi);
 	*charSize_s = gi.width >= gi.height ? gi.width+2 : gi.height+2;
 
-	XftTextExtents8(disp,win.font_e,str_e,strlen(str_e),&gi);
+	XftTextExtents8(disp,win->font_e,str_e,strlen(str_e),&gi);
 	*charSize_e = gi.width+4;
 
-	if(win.rubi_fs != NULL){
-		utf8 = (char *)euc2utf8(con,str,strlen(str));
-		XftTextExtentsUtf8(disp,win.font_rubi,utf8,strlen(utf8),&gi);
-		*charSize_r = gi.width >= gi.height ? gi.width : gi.height;
-	}
+	utf8 = (char *)euc2utf8(con,str,strlen(str));
+	XftTextExtentsUtf8(disp,win->font_rubi,utf8,strlen(utf8),&gi);
+	*charSize_r = gi.width >= gi.height ? gi.width : gi.height;
 
 	if(utf8) free(utf8);
 	iconv_close(con);
@@ -486,7 +580,7 @@ static int _xtext_get_antifont_size(Display *disp,my_window_t win,int *charSize,
 
 	return 0;
 }
-int xtext_get_charsize(Display *disp,my_window_t win,IBUF_T *l_head,char_data_t *ct)
+int xtext_get_charsize(Display *disp,my_window_t *win,IBUF_T *l_head,char_data_t *ct)
 {
 	IBUF_T *ib;
 	ITEXT_T *it;
@@ -499,16 +593,16 @@ int xtext_get_charsize(Display *disp,my_window_t win,IBUF_T *l_head,char_data_t
 	int type;
 
 
-	if(win.fs == NULL){
+	if(win->fs == NULL){
 		fprintf(stderr,"ERROR : Font False\n");
 		exit(1);
 	}
 
 #if debug
-fprintf(stderr,"ANTI : %d\n",win.anti);
+fprintf(stderr,"ANTI : %d\n",win->anti);
 #endif
 
-	if(win.anti == 0){
+	if(win->anti == 0){
 		_xtext_get_normalfont_size(win,&charSize,&charSize_s,&charSize_e,&charSize_r);
 
 	}else{
@@ -533,11 +627,11 @@ fprintf(stderr,"Get CharSize End\n");
 	xpad = ct->xpad;
 
 	one_line = charSize + xpad + charSize_r;
-	start_line = win.width - one_line;
+	start_line = win->width - one_line;
 	end_line = 0;
 
 	start_low = charSize * 2;
-	end_low = win.height - charSize;
+	end_low = win->height - charSize;
 
 	x = start_line;
 	y = start_low;
